       PROGRAM MAP_STEEL_MS
C
C This program uses the output from MTDATA to predict the change in free energy
C for the ferrite -> austenite transformation, and hence the Ms temperature
C using Ghosh and Olson's equation.
C
C Note modifications to data input procedures, added by S. Cardie 2/4/97
C
       IMPLICIT NONE
       DOUBLE PRECISION AR(100),WT(100),AT(100),ML(100),T60(100)
       DOUBLE PRECISION ZENER(100),EENEW(100),EEOLD(100)
       DOUBLE PRECISION C(100),D(100),E(100),F(100),G(100),T(100)
       DOUBLE PRECISION TC(100),H(100)
C
       DOUBLE PRECISION AA, BB, P, Y, YY, SUM
       DOUBLE PRECISION FD, FE, FF, FG, FH, FI, FJ
       DOUBLE PRECISION FFD, FFE, FFF, FFG, FFH, FFI, FFJ
       DOUBLE PRECISION L1, L2, L3, L4
       DOUBLE PRECISION MAX, MAXG, MIN, MING
       DOUBLE PRECISION ZZ1, ZZ2, ZZ3, ZZ11, ZZ22, ZZ33, ZEN1, ZEN2
C
       DOUBLE PRECISION ADD
C
       INTEGER I, J, J6, J7, K
C
C Read in initial values from 'ms-profile.dat'
C  Added by SC, 2/4/97
C
       CALL PROFILE(J6, C, Y, J7, T, D, E)
C 
C Initialise AR to convert from wt.% to mole fraction
C relative atomic masses of the elements in order
C C,Si,Mn,Ni,Mo,Cr,V,Cu,W,Nb,N,Co,Fe
C
       AR(1)  =  12.0111
       AR(2)  =  28.0855
       AR(3)  =  54.938
       AR(4)  =  58.71
       AR(5)  =  95.94
       AR(6)  =  51.996
       AR(7)  =  50.9415
       AR(8)  =  63.546
       AR(9)  = 183.85
       AR(10) =  92.9064
       AR(11) =  14.0067 
       AR(12) =  58.993
       AR(13) =  55.847
C
       DO 220, I = 1,13
        WT(I) = C(I)
220    CONTINUE
C
       WT(13) = 100.0D0-ADD(WT,13)
C
       DO 410, I  =1,13
        ML(I) = (WT(I))/AR(I)
410    CONTINUE
C
       SUM = ADD(ML,13)
C
       DO 11, J = 1,13
         AT(J) = (ML(J))/SUM
11     CONTINUE
C
       C(13) = WT(13)
C
       WRITE (6,*) 'Composition of sample',J6
       WRITE (6,997) 'Element ','C','Si','Mn','Ni','Mo','Cr',
     &                'V','Cu','W','Nb',
     &                'N','Co','Fe'
       WRITE (6,998) 'Wt%  ',(WT(I),I=1,13)
       WRITE (6,999) 'At fract. ',(AT(J),J=1,13)
C
997    FORMAT (A10,16A8)
998    FORMAT (A10,16F8.4)
999    FORMAT (A10,16F8.4) 
C
C This simply converts Kelvin to Centigrade
C
       DO 87, K = 1,J7
        TC(K) = T(K)-273.0
87     CONTINUE
C  
C This provides the conversion factor from kg to moles
C
       DO 99, K = 1,16
        H(K) = C(K)*AR(K)
        P    = P+H(K)
99     CONTINUE
C
C This calculates the change in free energy using the data
C in J/kg, and converts this value to J/mole using the conversion
C factor calculated above.
C
       DO 39 K=1,J7
        F(K)=D(K)-E(K)
        G(K)=(F(K))*P/100000.0
39     CONTINUE
C
C        WRITE (6,*) ' '
C        WRITE (6,*) 'Free energy due to composition'
C        WRITE (6,*) ' '
C
C This do loop is commented out in co13xbothk.for
C
       DO 49 K=1,J7
        WRITE (6,900) TC(K),G(K)
900     FORMAT (10X,F8.0,' Centigrade',10X,F10.2,' J/mol')
49     CONTINUE
C
       WRITE (6,*) ' '
C
C this bit calculates Olson's equation for the weight\% given
C
       FD=-(683.3+4009*AT(1)**(0.5)+1879*AT(2)**(0.5)+1980*AT(3)**(0.5))
       FF=-(172*AT(4)**(0.5))
       FG=-(1418*AT(5)**(0.5)+1868*AT(6)**(0.5)+1618*AT(7)**(0.5))
       FH=-(752*AT(8)**(0.5))
       FI=-(714*AT(9)**(0.5)+1653*AT(10)**(0.5))
       FJ=-(3097*AT(11)**(0.5))
       FE=FD+FF+FG+FH+FI+FJ
C
       WRITE (6,*) '***NEW K1***'
       WRITE (6,*) 'Predicted free energy due to comp = ',FE,' J/mol'
       WRITE (6,*) ' '
C
C This section is the Zener ordering term, as a function of Carbon
C
       DO 79 K=1,J7
        T60(K)=T(K)*(1-AT(1))/(28080*AT(1))
        IF (T60(K) .GT. 1.0) GOTO 14
        IF (T60(K) .GT. 0.25) GOTO 17
        ZZ1=0.2307+42.7974*T60(K)-233.8631*((T60(K))**2)
        ZZ2=645.4485*((T60(K))**3)-954.3995*((T60(K))**4)
        ZZ3=711.8095*((T60(K))**5)-211.5136*((T60(K))**6)
        ZEN1=ZZ1+ZZ2+ZZ3
        ZZ11=-2.6702+45.6337*T60(K)-225.3965*((T60(K))**2)
        ZZ22=567.7112*((T60(K))**3)-771.6466*((T60(K))**4)
        ZZ33=538.1778*((T60(K))**5)-151.3818*((T60(K))**6)
        ZEN2=ZZ11+ZZ22+ZZ33
        GOTO 16
17      ZEN2=1
        ZEN1=3.295
16      AA=((ZEN2*AT(1))**2)*(-50898.56)/(1-AT(1))
        BB=ZEN1*T60(K)*AT(1)*0.6623741
        ZENER(K)=AA+BB
        ZENER(K)=ZENER(K)*4.187
        GOTO 19
14      ZENER(K)=0
19      YY=1
79     CONTINUE
C
C       WRITE (6,*) ' '
C       WRITE (6,*) 'Total free energy'
C       WRITE (6,*) ' '
C
        DO 44 K=1,J7
         EENEW(K)=ZENER(K)+G(K)
44      CONTINUE
C
        WRITE (6,*) ' '
C
        WRITE (6,*) 'Without Zener ordering'
C
C This do loop calculates an approximate prediction for the Ms temperature, 
C on the basis of Olson's equation, and the MTDATA output.
C Assuming delta G (G(K)) and Ms free energy are equal, it also assumes a
C linearity between temperature/free energy data.
C A more accurate value may obviously be gained from plotting a graph of the 
C points.
C
        DO 18 K=1,(J7-1)
         IF (G(K) .GT. FE .AND. FE .GT. G(K+1)) THEN
C            WRITE(6,674) TC(K+1),TC(K)
C674         FORMAT (10X,F8.0,' C < Ms < ',F8.0,' C')
            MIN=TC(K+1)
            MAX=TC(K)
            MING=G(K+1)
            MAXG=G(K)
         END  IF
18      CONTINUE
C
        L1=MIN+ ((FE-MING)*(MAX-MIN)/(MAXG-MING))
        WRITE (6,*) ' '
        WRITE (6,675) L1
675     FORMAT (10X,'Ms (new) approximately ',F8.0,' Centigrade')
C
        WRITE(6,*) ' '
        WRITE(6,*) 'With Zener ordering'
C
C This do loop calculates an approximate prediction for the Ms temperature, 
C on the basis of Olson's equation, and the MTDATA output. It also incorporates 
C the Zener ordering term.
C Assuming delta G (G(K)) and Ms free energy are equal, it also assumes a
C linearity between temperature/free energy data.
C A more accurate value may obviously be gained from plotting a graph of the
C points.
C
        DO 73 K=1,(J7-1)
         IF (EENEW(K) .GT. FE .AND. FE .GT. EENEW(K+1)) THEN
C             WRITE(6,682) TC(K+1),TC(K)
C682          FORMAT (10X,F8.0,' C < Ms < ',F8.0,' C') 
            MIN=TC(K+1)
            MAX=TC(K)
            MING=EENEW(K+1)
            MAXG=EENEW(K)
         END  IF
73      CONTINUE
C
        L2=MIN+ ((FE-MING)*(MAX-MIN)/(MAXG-MING))
C
        WRITE (6,*) ' '
        WRITE (6,678) L2
678     FORMAT (10X,'Ms (new) approximately ',F8.0,' Centigrade')
C
        WRITE(6,*) ' '
C
C this bit calculates Olson's equation for the weight% given with the old K1
C
        FFD=-(1010+4009*AT(1)**(0.5)+1879*AT(2)**(0.5)
     &                              +1980*AT(3)**(0.5))
        FFF=-(172*AT(4)**(0.5))
        FFG=-(1418*AT(5)**(0.5)+1868*AT(6)**(0.5)
     &                         +1618*AT(7)**(0.5))
        FFH=-(752*AT(8)**(0.5))
        FFI=-(714*AT(9)**(0.5)+1653*AT(10)**(0.5))
        FFJ=-(3097*AT(11)**(0.5))
        FFE=FFD+FFF+FFG+FFH+FFI+FFJ
C
        WRITE (6,*) '***OLD K1***'
        WRITE (6,*) 'Predicted free energy due to comp = ',FFE,' J/mol'
        WRITE (6,*) '  '
C
        DO 444 K=1,J7
         EEOLD(K)=ZENER(K)+G(K)
444     CONTINUE
C
        WRITE (6,*) ' '
        WRITE (6,*) 'Without Zener ordering'
C
C This do loop calculates an approximate prediction for the Ms temperature, 
C on the basis of Olson's equation, and the MTDATA output.
C Assuming delta G (G(K)) and Ms free energy are equal, it also assumes a
C linearity between temperature/free energy data.
C A more accurate value may obviously be gained from plotting a graph of the 
C points.
C
        DO 118 K=1,(J7-1)
         IF (G(K) .GT. FFE .AND. FFE .GT. G(K+1)) THEN
C            WRITE(6,675) TC(K+1),TC(K)
C675         FORMAT (10X,F8.0,' C < Ms < ',F8.0,' C') 
            MIN=TC(K+1)
            MAX=TC(K)
            MING=G(K+1)
            MAXG=G(K)
         END  IF
118     CONTINUE
C
        L3=MIN+ ((FFE-MING)*(MAX-MIN)/(MAXG-MING))
C
        WRITE (6,*) ' '
        WRITE (6,676) L3
676     FORMAT (10X,'Ms (old) approximately ',F8.0,' Centigrade')
C
        WRITE(6,*) ' '
        WRITE(6,*) 'With Zener ordering'
C
C This do loop calculates an approximate prediction for the Ms temperature, 
C on the basis of Olson's equation, and the MTDATA output. It also incorporates 
C the Zener ordering term.
C Assuming delta G (G(K)) and Ms free energy are equal, it also assumes a
C linearity between temperature/free energy data.
C A more accurate value may obviously be gained from plotting a graph of the 
C points.
C
       DO 773 K=1,(J7-1)
        IF (EEOLD(K) .GT. FFE .AND. FFE .GT. EEOLD(K+1)) THEN
C           WRITE(6,682) TC(K+1),TC(K)
C682        FORMAT (10X,F8.0,' C < Ms < ',F8.0,' C') 
           MIN=TC(K+1)
           MAX=TC(K)
           MING=EEOLD(K+1)
           MAXG=EEOLD(K)
        ENDIF
773    CONTINUE
C
        L4=MIN+ ((FFE-MING)*(MAX-MIN)/(MAXG-MING))
C
        WRITE (6,*) ' '
        WRITE (6,679) L4
679     FORMAT (10X,'Ms (old) approximately ',F8.0,' Centigrade')
C
        WRITE(6,*) ' '
C
C this bit prints out whether there is an experimental value or not
C
        IF (Y .LT. 0.1D+00) THEN
           WRITE(6,*) 'No experimental data was provided for comparison'
        ENDIF
C
        IF (Y .GT. 0.0D+00) THEN 
           WRITE(6,994) Y
994        FORMAT('Experimental Value  ',10X,F10.2,' Centigrade') 
        ENDIF
C
        STOP
        END
C
C-----------------------------------------------------------------------
C
       SUBROUTINE MAP_UTIL_REED(A)
       IMPLICIT NONE
       DOUBLE PRECISION A
C
996    READ(*,*,ERR=999)A
         GOTO 998
999      WRITE(*,997)
997      FORMAT(19X,' Incorrect Input. Try again')
         GOTO 996
C
998    RETURN
       END
C
C-----------------------------------------------------------------------
C
       SUBROUTINE MAP_UTIL_REEDI(I)
       IMPLICIT NONE
       INTEGER I
C
996    READ(*,*,ERR=999)I
          GOTO 998
999       WRITE(*,997)
997       FORMAT(19X,' Incorrect Input. Try again')
          GOTO 996
C
998    RETURN
       END
C
C-----------------------------------------------------------------------
C
        SUBROUTINE BOUND(A,B,C,I)
        IMPLICIT NONE
C
        DOUBLE PRECISION A,B,C
        INTEGER I
        CHARACTER*12 ELEMENT(12)
C
        DATA ELEMENT(1)/'CARBON'/,
     &       ELEMENT(2)/'SILICON'/
     &       ELEMENT(3)/'MANGANESE'/
     &       ELEMENT(4)/'NICKEL'/
     &       ELEMENT(5)/'MOLYBDENUM'/
     &       ELEMENT(6)/'CHROMIUM'/
     &       ELEMENT(7)/'VANADIUM'/
     &       ELEMENT(8)/'COPPER'/
     &       ELEMENT(9)/'TUNGSTEN'/
     &       ELEMENT(10)/'NIOBIUM'/
     &       ELEMENT(11)/'NITROGEN'/
     &       ELEMENT(12)/'COBALT'/
C
        WRITE(6,*) 'Element: ', ELEMENT(I)
2       CONTINUE
        WRITE(6,1000) B,C
1000    FORMAT(12X,' Input value out of bounds'/,
     &  12X,'The limits are ',D12.4,' to ', D12.4)
C
        CALL MAP_UTIL_REED(A)
C
        IF(A .LT. B .OR. A .GT. C)  GOTO 2
C
        RETURN
        END
C
C-----------------------------------------------------------------------
C
        DOUBLE PRECISION FUNCTION ADD(X,I)
C
        IMPLICIT NONE
C
        INTEGER I, I1
        DOUBLE PRECISION A, X(100)
C
        A = 0.0D0
C
        DO 12, I1 = 1,I
         A = A+X(I1)
12      CONTINUE
C
        ADD = A
C
        RETURN
        END
C
C-----------------------------------------------------------------------
C PROFILE : reads from file 'ms-profile.dat'
C  Added by SC 2/4/97
C
       subroutine profile(J6, C, Y, J7, T, D, E)
       implicit none
       double precision c(100), d(100), e(100), t(100)
       double precision y, ubound, lbound
       integer i, j6, j7
C
       open(11,file='ms-profile.dat')
C
C Identification Number
C
       read(11,*) J6
       if (j6 .le. 0)  call MAP_UTIL_REEDI(J6)
C
C Read elements 1 to 12 as defined in documentation
C  and check for upper and lower bounds.  Offer manual
C  entry if out-of-bounds
C
       do 10, i = 1, 12
        read(11,*) C(i)
        if (i .eq. 1)  then      ! Carbon
          ubound = 2.0D+00
          lbound = 0.1D-03
        else if (i .eq. 2)  then ! Silicon
          ubound = 3.5D+00
          lbound = 0.0D+00
        else if (i .eq. 3)  then ! Manganese
          ubound = 5.0D+00
          lbound = 0.0D+00
        else if (i .eq. 4)  then ! Nickel
          ubound = 6.0D+00
          lbound = 0.0D+00
        else if (i .eq. 5)  then ! Molybdenum
          ubound = 5.6D+00
          lbound = 0.0D+00
        else if (i .eq. 6)  then ! Chromium
          ubound = 15.5D+00
          lbound = 0.0D0+00
        else if (i .eq. 7)  then ! Vanadium
          ubound = 1.80D+00
          lbound = 0.0D0+00
        else if (i .eq. 8)  then ! Copper
          ubound = 1.50D+00
          lbound = 0.0D0+00
        else if (i .eq. 9)  then ! Tungsten
          ubound = 18.0D+00
          lbound = 0.0D0+00
        else if (i .eq. 10) then ! Niobium
          ubound = 1.50D+00
          lbound = 0.0D0+00
        else if (i .eq. 11) then ! Nitrogen
          ubound = 1.50D+00
          lbound = 0.0D0+00
        else if (i .eq. 12) then ! Cobalt
          ubound = 1.50D+00
          lbound = 0.0D0+00
        else                     ! Anything else
          ubound = 1.50D+00
          lbound = 0.0D0+00
        end  if
C
        if(C(i) .LT. lbound .OR. C(i) .GT. ubound)
     &              call BOUND(C(i),lbound,ubound, i)
10     continue
C
C Get Y value
C
       read(11,*) Y
C
C Get no. of steps in profile
C
       read(11,*) J7
       if (j7 .le. 0)  call MAP_UTIL_REEDI(J7)
C
C Read in T, D and E values
C
       do 15, i = 1, J7
        read(11,*) T(i), D(i), E(i)
15     continue
C
       close(11)
       return
       end
C
C-----------------------------------------------------------------------
C
